@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2003, Index Data.
+ * Copyright (c) 1995-2004, Index Data.
  *
  * Permission to use, copy, modify, distribute, and sell this software and
  * its documentation, in whole or in part, for any purpose, is hereby granted,
@@ -23,7 +23,7 @@
  * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
  * OF THIS SOFTWARE.
  *
- * $Id: backend.h,v 1.22 2003/12/29 14:54:33 adam Exp $
+ * $Id: backend.h,v 1.25 2004/04/30 19:10:35 adam Exp $
  */
 
 #ifndef BACKEND_H
@@ -191,6 +191,7 @@ typedef struct {
     ODR print;
     char *explain_buf;
     char *database;
+    char *schema;
 } bend_explain_rr;
 
 typedef struct bend_initrequest
@@ -272,6 +273,9 @@ typedef struct statserv_options_block
     char service_display_name[128]; /* The service display name */
 #endif /* WIN32 */
     struct bend_soap_handler *soap_handlers;
+    char pid_fname[128];            /* pid fname */
+    int background;                 /* auto daemon */
+    char cert_fname[128];           /* SSL certificate fname */
 } statserv_options_block;
 
 YAZ_EXPORT int statserv_main(
