@@ -1,9 +1,8 @@
 /*
- * Copyright (c) 1998-2003, Index Data
+ * Copyright (c) 1998-2004, Index Data
  * See the file LICENSE for details.
- * Sebastian Hammer, Adam Dickmeiss
  *
- * $Id: proto.h,v 1.11 2003/07/18 19:54:30 mike Exp $
+ * $Id: proto.h,v 1.15 2004/05/10 07:52:07 adam Exp $
  */
 #ifndef Z_PROTO_H
 #define Z_PROTO_H
@@ -47,6 +46,7 @@
 
 YAZ_BEGIN_CDECL
 
+#if 1
 #define Z_PRES_SUCCESS   Z_PresentStatus_success
 #define Z_PRES_PARTIAL_1 Z_PresentStatus_partial_1
 #define Z_PRES_PARTIAL_2 Z_PresentStatus_partial_2
@@ -96,6 +96,7 @@ YAZ_BEGIN_CDECL
 #define Z_SortResultSetStatus_interim     Z_SortResponse_interim
 #define Z_SortResultSetStatus_unchanged   Z_SortResponse_unchanged
 #define Z_SortResultSetStatus_none        Z_SortResponse_none
+#endif
 
 typedef Z_External Z_DatabaseRecord;
 typedef struct Z_IOItemOrder Z_ItemOrder;
@@ -114,6 +115,12 @@ YAZ_EXPORT void yaz_display_grs1(WRBUF wrbuf, Z_GenericRecord *r, int flags);
 
 YAZ_EXPORT void yaz_display_OPAC(WRBUF wrbuf, Z_OPACRecord *r, int flags);
 
+YAZ_EXPORT int yaz_init_opt_encode(Z_Options *opt, const char *opt_str,
+				   int *error_pos);
+YAZ_EXPORT void yaz_init_opt_decode(Z_Options *opt,
+				    void (*pr)(const char *name,
+					       void *clientData),
+				    void *clientData);
 YAZ_END_CDECL
 
 #include <yaz/prt-ext.h>
