@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2002-2003, Index Data.
+ * Copyright (c) 2002-2004, Index Data.
  * See the file LICENSE for details.
  *
- * $Id: srw.h,v 1.11 2004/01/05 09:34:42 adam Exp $
+ * $Id: srw.h,v 1.16 2004/02/14 15:58:42 adam Exp $
  */
 
 #ifndef YAZ_SRW_H
@@ -24,8 +24,9 @@ typedef struct {
 } Z_SRW_record;
 
 typedef struct {
-    int  *code;
+    char *uri;
     char *details;
+    char *message;
 } Z_SRW_diagnostic;
     
 typedef struct {
@@ -75,6 +76,7 @@ typedef struct {
 typedef struct {
     char *recordPacking;
     char *database;
+    char *stylesheet;
 } Z_SRW_explainRequest;
 
 typedef struct {
@@ -141,7 +143,16 @@ YAZ_EXPORT void yaz_uri_val_int(const char *path, const char *name,
 YAZ_EXPORT int yaz_srw_decode(Z_HTTP_Request *hreq, Z_SRW_PDU **srw_pdu,
 			      Z_SOAP **soap_package, ODR decode, char **charset);
 YAZ_EXPORT int yaz_sru_decode(Z_HTTP_Request *hreq, Z_SRW_PDU **srw_pdu,
-			      Z_SOAP **soap_package, ODR decode, char **charset);
+			      Z_SOAP **soap_package, ODR decode, 
+			      char **charset,
+			      Z_SRW_diagnostic **, int *num_diagnostic);
+
+YAZ_EXPORT void yaz_add_srw_diagnostic(ODR o, Z_SRW_diagnostic **d,
+				       int *num, int code,
+				       const char *addinfo);
+    
+YAZ_EXPORT void yaz_mk_std_diagnostic(ODR o, Z_SRW_diagnostic *d, 
+				      int code, const char *details);
 YAZ_END_CDECL
 
 #endif
