@@ -1,8 +1,9 @@
 /*
  * Public header for ZOOM C.
- * $Id: zoom.h,v 1.19 2003/11/19 19:07:26 adam Exp $
+ * $Id: zoom.h,v 1.23 2004/04/28 22:44:59 adam Exp $
  */
 
+#include <stdlib.h>
 #include <yaz/yconfig.h>
 
 #define ZOOM_BEGIN_CDECL YAZ_BEGIN_CDECL
@@ -59,6 +60,10 @@ ZOOM_API(void)
 ZOOM_connection_option_set (ZOOM_connection c, const char *key,
                             const char *val);
 
+ZOOM_API(void)
+ZOOM_connection_option_setl (ZOOM_connection c, const char *key,
+			     const char *val, int len);
+
 /* return error code (0 == success, failure otherwise). cp
    holds error string on failure, addinfo holds addititional info (if any)
 */
@@ -94,6 +99,7 @@ ZOOM_diag_str (int error);
 #define ZOOM_ERROR_TIMEOUT 10007
 #define ZOOM_ERROR_UNSUPPORTED_PROTOCOL 10008
 #define ZOOM_ERROR_UNSUPPORTED_QUERY 10009
+#define ZOOM_ERROR_INVALID_QUERY 10010
 
 ZOOM_API(int)
 ZOOM_connection_last_event(ZOOM_connection cs);
@@ -146,6 +152,10 @@ ZOOM_resultset_record (ZOOM_resultset s, size_t pos);
 ZOOM_API(ZOOM_record)
 ZOOM_resultset_record_immediate (ZOOM_resultset s, size_t pos);
 
+/* reset record cache for result set */
+ZOOM_API(void)
+ZOOM_resultset_cache_reset(ZOOM_resultset r);
+
 /* ----------------------------------------------------------- */
 /* records */
 
@@ -224,6 +234,12 @@ ZOOM_API(void)
     ZOOM_package_option_set (ZOOM_package p, const char *key,
                              const char *val);
 
+/* ----------------------------------------------------------- */
+/* Sort */
+ZOOM_API(void)
+    ZOOM_resultset_sort(ZOOM_resultset r,
+			const char *sort_type, const char *sort_spec);
+
 /* ----------------------------------------------------------- */
 /* options */
 
