@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: comstack.c,v 1.2 2003/11/17 10:40:56 mike Exp $
+ * $Id: comstack.c,v 1.10 2004/05/03 09:00:50 adam Exp $
  */
 
 #include <string.h>
@@ -14,6 +14,10 @@
 #include <yaz/unix.h>
 #include <yaz/odr.h>
 
+#ifdef WIN32
+#define strncasecmp _strnicmp
+#endif
+
 static const char *cs_errlist[] =
 {
     "No error or unspecified error",
@@ -97,8 +101,8 @@ COMSTACK cs_create_host(const char *type_and_host, int blocking, void **vp)
     {
 	t = tcpip_type;
         host = type_and_host + 5;
-        if (host[0] == '/' && host[1] == '/')
-            host = host + 2;
+        while (host[0] == '/')
+            host++;
         proto = PROTO_HTTP;
     }
     else if (strncmp(type_and_host, "https:", 6) == 0)
@@ -106,8 +110,8 @@ COMSTACK cs_create_host(const char *type_and_host, int blocking, void **vp)
 #if HAVE_OPENSSL_SSL_H
 	t = ssl_type;
         host = type_and_host + 6;
-        if (host[0] == '/' && host[1] == '/')
-            host = host + 2;
+        while (host[0] == '/')
+            host++;
         proto = PROTO_HTTP;
 #else
 	return 0;
@@ -117,7 +121,6 @@ COMSTACK cs_create_host(const char *type_and_host, int blocking, void **vp)
     {
 	t = tcpip_type;
 	host = type_and_host;
-        
     }
     cs = cs_create (t, blocking, proto);
     if (!cs)
@@ -136,6 +139,7 @@ int cs_look (COMSTACK cs)
     return cs->event;
 }
 
+#define CHUNK_DEBUG 0
 int cs_complete_auto(const unsigned char *buf, int len)
 {
     if (len > 5 && buf[0] >= 0x20 && buf[0] < 0x7f
@@ -156,23 +160,26 @@ int cs_complete_auto(const unsigned char *buf, int len)
                 {
                     if (chunked)
                     { 
+			/* inside chunked body .. */
                         while(1)
                         {
-                            int chunk_len = 0;
+                            int j, chunk_len = 0;
                             i += 2;
-#if 0
+#if CHUNK_DEBUG
 /* debugging */
                             if (i <len-2)
                             {
-                                printf ("\n>>>");
+                                printf ("\n<<<");
+				int j;
                                 for (j = i; j <= i+4; j++)
                                     printf ("%c", buf[j]);
-                                printf ("<<<\n");
+                                printf (">>>\n");
                             }
 #endif
+			    /* read chunk length */
                             while (1)
                                 if (i >= len-2) {
-#if 0
+#if CHUNK_DEBUG
 /* debugging */                                    
                                     printf ("XXXXXXXX not there yet 1\n");
                                     printf ("i=%d len=%d\n", i, len);
@@ -189,29 +196,41 @@ int cs_complete_auto(const unsigned char *buf, int len)
                                         (buf[i++] - ('a'-10));
                                 else
                                     break;
-                            if (buf[i] != '\r' || buf[i+1] != '\n' ||
-                                chunk_len < 0)
-                                return i+2;    /* bad. stop now */
+			    /* move forward until CRLF - skip chunk ext */
+			    j = 0;
+			    while (buf[i] != '\r' && buf[i+1] != '\n')
+			    {
+				if (i >= len-2)
+				    return 0;   /* need more buffer .. */
+				if (++j > 1000)
+				    return i; /* enough.. stop */
+				i++;
+			    }
+			    /* got CRLF */
+#if CHUNK_DEBUG
+			    printf ("XXXXXX chunk_len=%d\n", chunk_len);
+#endif			    
+			    if (chunk_len < 0)
+				return i+2;    /* bad chunk_len */
                             if (chunk_len == 0)
-                            {
-                                /* consider trailing headers .. */
-                                while(i <= len-4)
-                                {
-                                    if (buf[i] == '\r' &&  buf[i+1] == '\n' &&
-                                        buf[i+2] == '\r' && buf[i+3] == '\n')
-                                        if (len >= i+4)
-                                            return i+4;
-                                    i++;
-                                }
-#if 0
-/* debugging */
-                                printf ("XXXXXXXXX not there yet 2\n");
-                                printf ("i=%d len=%d\n", i, len);
-#endif
-                                return 0;
-                            }
+				break;
                             i += chunk_len+2;
                         }
+			/* consider trailing headers .. */
+			while(i <= len-4)
+			{
+			    if (buf[i] == '\r' &&  buf[i+1] == '\n' &&
+				buf[i+2] == '\r' && buf[i+3] == '\n')
+				if (len >= i+4)
+				    return i+4;
+			    i++;
+			}
+#if CHUNK_DEBUG
+/* debugging */
+			printf ("XXXXXXXXX not there yet 2\n");
+			printf ("i=%d len=%d\n", i, len);
+#endif
+			return 0;
                     }
                     else
                     {   /* not chunked ; inside body */
@@ -222,21 +241,21 @@ int cs_complete_auto(const unsigned char *buf, int len)
                     }
                     break;
                 }
-                else if (i < len - 21 &&
-                         !memcmp(buf+i, "Transfer-Encoding: ", 18))
+                else if (i < len - 20 && 
+                         !strncasecmp((const char *) buf+i, "Transfer-Encoding:", 18))
                 {
                     i+=18;
-                    if (buf[i] == ' ')
+                    while (buf[i] == ' ')
                         i++;
                     if (i < len - 8)
-                        if (!memcmp(buf+i, "chunked", 7))
+                        if (!strncasecmp((const char *) buf+i, "chunked", 7))
                             chunked = 1;
                 }
-                else if (i < len - 18 &&
-                         !memcmp(buf+i, "Content-Length: ", 15))
+                else if (i < len - 17 &&
+                         !strncasecmp((const char *)buf+i, "Content-Length:", 15))
                 {
                     i+= 15;
-                    if (buf[i] == ' ')
+                    while (buf[i] == ' ')
                         i++;
                     content_len = 0;
                     while (i <= len-4 && isdigit(buf[i]))
