@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: seshigh.c,v 1.13 2004/01/05 09:34:42 adam Exp $
+ * $Id: seshigh.c,v 1.24 2004/05/10 07:48:56 adam Exp $
  */
 
 /*
@@ -471,6 +471,7 @@ static int srw_bend_init(association *assoc)
     ce = yaz_set_proposal_charneg(assoc->decode, &encoding, 1, 0, 0, 1);
     assoc->init->charneg_request = ce->u.charNeg3;
 #endif
+    assoc->backend = 0;
     if (!(binitres = (*cb->bend_init)(assoc->init)))
     {
     	yaz_log(LOG_WARN, "Bad response from backend.");
@@ -564,12 +565,8 @@ static void srw_bend_search(association *assoc, request *req,
         {
             srw_error = 3;  /* assume Authentication error */
 
-            srw_res->num_diagnostics = 1;
-            srw_res->diagnostics = (Z_SRW_diagnostic *)
-                odr_malloc(assoc->encode, sizeof(*srw_res->diagnostics));
-            srw_res->diagnostics[0].code = 
-                odr_intdup(assoc->encode, srw_error);
-            srw_res->diagnostics[0].details = 0;
+	    yaz_add_srw_diagnostic(assoc->encode, &srw_res->diagnostics,
+				   &srw_res->num_diagnostics, 1, 0);
             return;
         }
     }
@@ -635,9 +632,8 @@ static void srw_bend_search(association *assoc, request *req,
         srw_res->num_diagnostics = 1;
         srw_res->diagnostics = (Z_SRW_diagnostic *)
 	    odr_malloc(assoc->encode, sizeof(*srw_res->diagnostics));
-        srw_res->diagnostics[0].code = 
-            odr_intdup(assoc->encode, srw_error);
-        srw_res->diagnostics[0].details = 0;
+	yaz_mk_std_diagnostic(assoc->encode,
+			      srw_res->diagnostics, srw_error, 0);
         return;
     }
     
@@ -665,13 +661,11 @@ static void srw_bend_search(association *assoc, request *req,
         srw_res->num_diagnostics = 1;
         srw_res->diagnostics = (Z_SRW_diagnostic *)
 	    odr_malloc(assoc->encode, sizeof(*srw_res->diagnostics));
-        srw_res->diagnostics[0].code = 
-            odr_intdup(assoc->encode, 
-                       yaz_diag_bib1_to_srw (rr.errcode));
-        srw_res->diagnostics[0].details = rr.errstring;
-        yaz_log(LOG_DEBUG, "srw_bend_search returned SRW error %d",
-                *srw_res->diagnostics[0].code);
-                
+	yaz_mk_std_diagnostic(assoc->encode, srw_res->diagnostics,
+			      yaz_diag_bib1_to_srw (rr.errcode),
+			      rr.errstring);
+        yaz_log(LOG_DEBUG, "srw_bend_search returned SRW error %s",
+                srw_res->diagnostics[0].uri);
     }
     else
     {
@@ -717,10 +711,11 @@ static void srw_bend_search(association *assoc, request *req,
                         srw_res->diagnostics = (Z_SRW_diagnostic *)
                             odr_malloc(assoc->encode, 
                                        sizeof(*srw_res->diagnostics));
-                        srw_res->diagnostics[0].code = 
-                            odr_intdup(assoc->encode, 
-                                       yaz_diag_bib1_to_srw (errcode));
-                        srw_res->diagnostics[0].details = rr.errstring;
+
+			yaz_mk_std_diagnostic(assoc->encode, 
+					      srw_res->diagnostics,
+					      yaz_diag_bib1_to_srw (errcode),
+					      rr.errstring);
                         break;
                     }
                     if (srw_res->records[j].recordData_buf)
@@ -758,6 +753,7 @@ static void srw_bend_explain(association *assoc, request *req,
         rr.print = assoc->print;
         rr.explain_buf = 0;
 	rr.database = srw_req->database;
+	rr.schema = "http://explain.z3950.org/dtd/2.0/";
         (*assoc->init->bend_explain)(assoc->backend, &rr);
         if (rr.explain_buf)
         {
@@ -779,17 +775,36 @@ static void process_http_request(association *assoc, request *req)
 {
     Z_HTTP_Request *hreq = req->gdu_request->u.HTTP_Request;
     ODR o = assoc->encode;
-    int r;
+    int r = 2;  /* 2=NOT TAKEN, 1=TAKEN, 0=SOAP TAKEN */
     Z_SRW_PDU *sr = 0;
     Z_SOAP *soap_package = 0;
     Z_GDU *p = 0;
     char *charset = 0;
-    Z_HTTP_Response *hres;
+    Z_HTTP_Response *hres = 0;
     int keepalive = 1;
+    char *stylesheet = 0;
+    Z_SRW_diagnostic *diagnostic = 0;
+    int num_diagnostic = 0;
 
-    r = yaz_srw_decode(hreq, &sr, &soap_package, assoc->decode, &charset);
+    if (!strcmp(hreq->path, "/test")) 
+    {	
+	p = z_get_HTTP_Response(o, 200);
+	hres = p->u.HTTP_Response;
+	hres->content_buf = "1234567890\n";
+	hres->content_len = strlen(hres->content_buf);
+	r = 1;
+    }
+    if (r == 2)
+    {
+	r = yaz_srw_decode(hreq, &sr, &soap_package, assoc->decode, &charset);
+	yaz_log(LOG_DEBUG, "yaz_srw_decode returned %d", r);
+    }
     if (r == 2)  /* not taken */
-	r = yaz_sru_decode(hreq, &sr, &soap_package, assoc->decode, &charset);
+    {
+	r = yaz_sru_decode(hreq, &sr, &soap_package, assoc->decode, &charset,
+			   &diagnostic, &num_diagnostic);
+	yaz_log(LOG_DEBUG, "yaz_sru_decode returned %d", r);
+    }
     if (r == 0)  /* decode SRW/SRU OK .. */
     {
 	int http_code = 200;
@@ -797,22 +812,54 @@ static void process_http_request(association *assoc, request *req)
 	{
 	    Z_SRW_PDU *res =
 		yaz_srw_get(assoc->encode, Z_SRW_searchRetrieve_response);
-	    
-	    srw_bend_search(assoc, req, sr->u.request, res->u.response, 
-			    &http_code);
+
+	    stylesheet = sr->u.request->stylesheet;
+	    if (num_diagnostic)
+	    {
+		res->u.response->diagnostics = diagnostic;
+		res->u.response->num_diagnostics = num_diagnostic;
+	    }
+	    else
+	    {
+		srw_bend_search(assoc, req, sr->u.request, res->u.response, 
+				&http_code);
+	    }
 	    if (http_code == 200)
 		soap_package->u.generic->p = res;
 	}
 	else if (sr->which == Z_SRW_explain_request)
 	{
             Z_SRW_PDU *res = yaz_srw_get(o, Z_SRW_explain_response);
-            srw_bend_explain(assoc, req, sr->u.explain_request,
+	    stylesheet = sr->u.explain_request->stylesheet;
+	    if (num_diagnostic)
+	    {	
+		res->u.explain_response->diagnostics = diagnostic;
+		res->u.explain_response->num_diagnostics = num_diagnostic;
+	    }
+	    srw_bend_explain(assoc, req, sr->u.explain_request,
 			     res->u.explain_response, &http_code);
 	    if (http_code == 200)
 		soap_package->u.generic->p = res;
 	}
+	else if (sr->which == Z_SRW_scan_request)
+	{
+            Z_SRW_PDU *res = yaz_srw_get(o, Z_SRW_scan_response);
+	    stylesheet = sr->u.scan_request->stylesheet;
+	    if (num_diagnostic)
+	    {	
+		res->u.scan_response->diagnostics = diagnostic;
+		res->u.scan_response->num_diagnostics = num_diagnostic;
+	    }
+	    yaz_add_srw_diagnostic(o, 
+				   &res->u.scan_response->diagnostics,
+				   &res->u.scan_response->num_diagnostics,
+				   4, "scan");
+	    if (http_code == 200)
+		soap_package->u.generic->p = res;
+	}
 	else
 	{
+		 yaz_log(LOG_LOG, "generate soap error");
 	    http_code = 500;
 	    z_soap_error(assoc->encode, soap_package,
 			 "SOAP-ENV:Client", "Bad method", 0); 
@@ -832,9 +879,9 @@ static void process_http_request(association *assoc, request *req)
 	    int ret;
 	    p = z_get_HTTP_Response(o, 200);
 	    hres = p->u.HTTP_Response;
-	    ret = z_soap_codec_enc(assoc->encode, &soap_package,
-				   &hres->content_buf, &hres->content_len,
-				   soap_handlers, charset);
+	    ret = z_soap_codec_enc_xsl(assoc->encode, &soap_package,
+				       &hres->content_buf, &hres->content_len,
+				       soap_handlers, charset, stylesheet);
 	    hres->code = http_code;
 
 	    strcpy(ctype, "text/xml");
@@ -848,7 +895,8 @@ static void process_http_request(association *assoc, request *req)
 	else
 	    p = z_get_HTTP_Response(o, http_code);
     }
-    else
+
+    if (p == 0)
 	p = z_get_HTTP_Response(o, 500);
     hres = p->u.HTTP_Response;
     if (!strcmp(hreq->version, "1.0")) 
@@ -1145,10 +1193,12 @@ static Z_APDU *process_initRequest(association *assoc, request *reqb)
     {
         Z_CharSetandLanguageNegotiation *negotiation =
             yaz_get_charneg_record (req->otherInfo);
-        if (negotiation->which == Z_CharSetandLanguageNegotiation_proposal)
+        if (negotiation &&
+	    negotiation->which == Z_CharSetandLanguageNegotiation_proposal)
             assoc->init->charneg_request = negotiation;
     }
     
+    assoc->backend = 0;
     if (!(binitres = (*cb->bend_init)(assoc->init)))
     {
     	yaz_log(LOG_WARN, "Bad response from backend.");
@@ -1239,7 +1289,7 @@ static Z_APDU *process_initRequest(association *assoc, request *reqb)
     if (ODR_MASK_GET(req->protocolVersion, Z_ProtocolVersion_1))
     {
     	ODR_MASK_SET(resp->protocolVersion, Z_ProtocolVersion_1);
-	assoc->version = 2; /* 1 & 2 are equivalent */
+	assoc->version = 1; /* 1 & 2 are equivalent */
     }
     if (ODR_MASK_GET(req->protocolVersion, Z_ProtocolVersion_2))
     {
@@ -1271,7 +1321,7 @@ static Z_APDU *process_initRequest(association *assoc, request *reqb)
 		assoc->init->implementation_name,
 		odr_prepend(assoc->encode, "GFS", resp->implementationName));
 
-    version = odr_strdup(assoc->encode, "$Revision: 1.13 $");
+    version = odr_strdup(assoc->encode, "$Revision: 1.24 $");
     if (strlen(version) > 10)	/* check for unexpanded CVS strings */
 	version[strlen(version)-2] = '\0';
     resp->implementationVersion = odr_prepend(assoc->encode,
@@ -1366,6 +1416,7 @@ static Z_External *init_diagnostics(ODR odr, int error, char *addinfo)
 
     e->which = Z_DiagnosticFormat_s_defaultDiagRec;
     e->u.defaultDiagRec = justdiag(odr, error, addinfo);
+    e->message = 0;
     return x;
 }
 
@@ -1448,7 +1499,7 @@ static Z_Records *pack_records(association *a, char *setname, int start,
     records->u.databaseOrSurDiagnostics = reclist;
     reclist->num_records = 0;
     reclist->records = list;
-    *pres = Z_PRES_SUCCESS;
+    *pres = Z_PresentStatus_success;
     *num = 0;
     *next = 0;
 
@@ -1492,7 +1543,7 @@ static Z_Records *pack_records(association *a, char *setname, int start,
 	    if (!freq.surrogate_flag)
 	    {
 		char s[20];
-		*pres = Z_PRES_FAILURE;
+		*pres = Z_PresentStatus_failure;
 		/* for 'present request out of range',
                    set addinfo to record position if not set */
 		if (freq.errcode == 13 && freq.errstring == 0)
@@ -1515,13 +1566,14 @@ static Z_Records *pack_records(association *a, char *setname, int start,
 	    this_length = odr_total(a->encode) - total_length - dumped_records;
 	yaz_log(LOG_DEBUG, "  fetched record, len=%d, total=%d dumped=%d",
 	    this_length, total_length, dumped_records);
-	if (this_length + total_length > a->preferredMessageSize)
+	if (a->preferredMessageSize > 0 &&
+		this_length + total_length > a->preferredMessageSize)
 	{
 	    /* record is small enough, really */
 	    if (this_length <= a->preferredMessageSize && recno > start)
 	    {
 	    	yaz_log(LOG_DEBUG, "  Dropped last normal-sized record");
-		*pres = Z_PRES_PARTIAL_2;
+		*pres = Z_PresentStatus_partial_2;
 		break;
 	    }
 	    /* record can only be fetched by itself */
@@ -1641,7 +1693,7 @@ static Z_APDU *response_searchRequest(association *assoc, request *reqb,
     int *nulint = odr_intdup (assoc->encode, 0);
     bool_t *sr = odr_intdup(assoc->encode, 1);
     int *next = odr_intdup(assoc->encode, 0);
-    int *none = odr_intdup(assoc->encode, Z_RES_NONE);
+    int *none = odr_intdup(assoc->encode, Z_SearchResponse_none);
 
     apdu->which = Z_APDU_searchResponse;
     apdu->u.searchResponse = resp;
@@ -1787,7 +1839,7 @@ static Z_APDU *process_presentRequest(association *assoc, request *reqb,
 	if (bprr->errcode)
 	{
 	    resp->records = diagrec(assoc, bprr->errcode, bprr->errstring);
-	    *resp->presentStatus = Z_PRES_FAILURE;
+	    *resp->presentStatus = Z_PresentStatus_failure;
 	}
     }
     apdu = (Z_APDU *)odr_malloc (assoc->encode, sizeof(*apdu));
@@ -2001,7 +2053,7 @@ static Z_APDU *process_sortRequest(association *assoc, request *reqb,
     bsrr->stream = assoc->encode;
     bsrr->print = assoc->print;
 
-    bsrr->sort_status = Z_SortStatus_failure;
+    bsrr->sort_status = Z_SortResponse_failure;
     bsrr->errcode = 0;
     bsrr->errstring = 0;
     
