@@ -2,7 +2,7 @@
  * Copyright (c) 2002-2004, Index Data.
  * See the file LICENSE for details.
  *
- * $Id: srw.c,v 1.14 2004/01/06 11:21:04 adam Exp $
+ * $Id: srw.c,v 1.25 2004/03/17 18:43:35 adam Exp $
  */
 
 #include <yaz/srw.h>
@@ -26,7 +26,7 @@ static void add_XML_n(xmlNodePtr ptr, const char *elem, char *val, int len)
     }
 }
 
-xmlNodePtr add_xsd_string_n(xmlNodePtr ptr, const char *elem, char *val,
+xmlNodePtr add_xsd_string_n(xmlNodePtr ptr, const char *elem, const char *val,
 			    int len)
 {
     if (val)
@@ -39,20 +39,20 @@ xmlNodePtr add_xsd_string_n(xmlNodePtr ptr, const char *elem, char *val,
     return 0;
 }
 
-xmlNodePtr add_xsd_string(xmlNodePtr ptr, const char *elem, char *val)
+xmlNodePtr add_xsd_string(xmlNodePtr ptr, const char *elem, const char *val)
 {
     if (val)
-        return xmlNewChild(ptr, 0, elem, val);
+        return xmlNewTextChild(ptr, 0, elem, val);
     return 0;
 }
 
-static void add_xsd_integer(xmlNodePtr ptr, const char *elem, int *val)
+static void add_xsd_integer(xmlNodePtr ptr, const char *elem, const int *val)
 {
     if (val)
     {
         char str[30];
         sprintf(str, "%d", *val);
-        xmlNewChild(ptr, 0, elem, str);
+        xmlNewTextChild(ptr, 0, elem, str);
     }
 }
 
@@ -113,7 +113,7 @@ static int match_xsd_XML_n(xmlNodePtr ptr, const char *elem, ODR o,
     if (!match_element(ptr, elem))
         return 0;
     ptr = ptr->children;
-    while (ptr && ptr->type != XML_TEXT_NODE && ptr->type != XML_COMMENT_NODE)
+    while (ptr && (ptr->type == XML_TEXT_NODE || ptr->type == XML_COMMENT_NODE))
 	ptr = ptr->next;
     if (!ptr)
         return 0;
@@ -285,8 +285,9 @@ static int yaz_srw_diagnostics(ODR o, xmlNodePtr pptr, Z_SRW_diagnostic **recs,
         *recs = odr_malloc(o, *num * sizeof(**recs));
 	for (i = 0; i < *num; i++)
 	{
-            (*recs)[i].code = 0;
+            (*recs)[i].uri = 0;
             (*recs)[i].details = 0;
+            (*recs)[i].message = 0;
 	} 
         for (i = 0, ptr = pptr->children; ptr; ptr = ptr->next)
         {
@@ -294,16 +295,20 @@ static int yaz_srw_diagnostics(ODR o, xmlNodePtr pptr, Z_SRW_diagnostic **recs,
                 !strcmp(ptr->name, "diagnostic"))
             {
                 xmlNodePtr rptr;
-                (*recs)[i].code = 0;
+                (*recs)[i].uri = 0;
                 (*recs)[i].details = 0;
+                (*recs)[i].message = 0;
                 for (rptr = ptr->children; rptr; rptr = rptr->next)
                 {
-                    if (match_xsd_integer(rptr, "code", o, 
-                                               &(*recs)[i].code))
+                    if (match_xsd_string(rptr, "uri", o, 
+					 &(*recs)[i].uri))
                         ;
                     else if (match_xsd_string(rptr, "details", o, 
                                               &(*recs)[i].details))
                         ;
+                    else if (match_xsd_string(rptr, "message", o, 
+                                              &(*recs)[i].message))
+                        ;
                 }
                 i++;
             }
@@ -313,11 +318,22 @@ static int yaz_srw_diagnostics(ODR o, xmlNodePtr pptr, Z_SRW_diagnostic **recs,
     {
         int i;
 	xmlNsPtr ns_diag =
-	    xmlNewNs(pptr, "http://www.loc.gov/zing/srw/diagnostics/", "diag");
+	    xmlNewNs(pptr, "info:srw/schema/1/diagnostic-v1.1", 0);
         for (i = 0; i < *num; i++)
         {
+	    const char *std_diag = "info:srw/diagnostic/1/";
             xmlNodePtr rptr = xmlNewChild(pptr, ns_diag, "diagnostic", 0);
-            add_xsd_integer(rptr, "code", (*recs)[i].code);
+            add_xsd_string(rptr, "uri", (*recs)[i].uri);
+	    if ((*recs)[i].message)
+		add_xsd_string(rptr, "message", (*recs)[i].message);
+	    else if ((*recs)[i].uri && 
+		     !strncmp((*recs)[i].uri, std_diag, strlen(std_diag)))
+	    {
+		int no = atoi((*recs)[i].uri + strlen(std_diag));
+		const char *message = yaz_diag_srw_str(no);
+		if (message)
+		    add_xsd_string(rptr, "message", message);
+	    }
             add_xsd_string(rptr, "details", (*recs)[i].details);
         }
     }
@@ -433,7 +449,10 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 
             for (; ptr; ptr = ptr->next)
             {
-                if (match_xsd_string(ptr, "query", o, 
+		if (match_xsd_string(ptr, "version", o,
+				     &(*p)->srw_version))
+                    ;
+                else if (match_xsd_string(ptr, "query", o, 
                                      &req->query.cql))
                     req->query_type = Z_SRW_query_type_cql;
                 else if (match_xsd_string(ptr, "pQuery", o, 
@@ -442,36 +461,33 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                 else if (match_xsd_string(ptr, "xQuery", o, 
                                      &req->query.xcql))
                     req->query_type = Z_SRW_query_type_xcql;
-                else if (match_xsd_string(ptr, "sortKeys", o, 
-                                          &req->sort.sortKeys))
-                    req->sort_type = Z_SRW_sort_type_sort;
-                else if (match_xsd_string(ptr, "recordSchema", o, 
-                                          &req->recordSchema))
-                    ;
-                else if (match_xsd_string(ptr, "recordPacking", o,
-                                          &req->recordPacking))
-                    ;
-                else if (match_xsd_string(ptr, "recordXPath", o,
-                                          &req->recordXPath))
-                    ;
                 else if (match_xsd_integer(ptr, "startRecord", o,
                                            &req->startRecord))
                     ;
                 else if (match_xsd_integer(ptr, "maximumRecords", o,
                                            &req->maximumRecords))
                     ;
+                else if (match_xsd_string(ptr, "recordPacking", o,
+                                          &req->recordPacking))
+                    ;
+                else if (match_xsd_string(ptr, "recordSchema", o, 
+                                          &req->recordSchema))
+                    ;
+                else if (match_xsd_string(ptr, "recordXPath", o,
+                                          &req->recordXPath))
+                    ;
+                else if (match_xsd_string(ptr, "resultSetTTL", o,
+                                           &req->database))
+                    ;
+                else if (match_xsd_string(ptr, "sortKeys", o, 
+                                          &req->sort.sortKeys))
+                    req->sort_type = Z_SRW_sort_type_sort;
                 else if (match_xsd_string(ptr, "stylesheet", o,
                                            &req->stylesheet))
                     ;
                 else if (match_xsd_string(ptr, "database", o,
                                            &req->database))
                     ;
-                else if (match_xsd_string(ptr, "resultSetTTL", o,
-                                           &req->database))
-                    ;
-                else if (match_xsd_string(ptr, "version", o,
-                                           &(*p)->srw_version))
-                    ;
                 /* missing is xQuery, xSortKeys .. */
             }
         }
@@ -494,7 +510,10 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 
             for (; ptr; ptr = ptr->next)
             {
-                if (match_xsd_integer(ptr, "numberOfRecords", o, 
+                if (match_xsd_string(ptr, "version", o,
+				     &(*p)->srw_version))
+                    ;
+                else if (match_xsd_integer(ptr, "numberOfRecords", o, 
                                       &res->numberOfRecords))
                     ;
                 else if (match_xsd_string(ptr, "resultSetId", o, 
@@ -507,16 +526,13 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                     yaz_srw_records(o, ptr, &res->records,
                                     &res->num_records, client_data,
                                     ns);
+                else if (match_xsd_integer(ptr, "nextRecordPosition", o,
+                                           &res->nextRecordPosition))
+                    ;
                 else if (match_element(ptr, "diagnostics"))
                     yaz_srw_diagnostics(o, ptr, &res->diagnostics,
                                         &res->num_diagnostics,
                                         client_data, ns);
-                else if (match_xsd_integer(ptr, "nextRecordPosition", o,
-                                           &res->nextRecordPosition))
-                    ;
-                else if (match_xsd_string(ptr, "version", o,
-                                           &(*p)->srw_version))
-                    ;
             }
         }
         else if (!strcmp(method->name, "explainRequest"))
@@ -528,17 +544,21 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
             req = (*p)->u.explain_request = odr_malloc(o, sizeof(*req));
 	    req->recordPacking = 0;
 	    req->database = 0;
+	    req->stylesheet = 0;
 	    for (; ptr; ptr = ptr->next)
 	    {
-		if (match_xsd_string(ptr, "database", o,
-				     &req->database))
-                    ;
+                if (match_xsd_string(ptr, "version", o,
+                                           &(*p)->srw_version))
+		    ;
+		else if (match_xsd_string(ptr, "stylesheet", o,
+					  &req->stylesheet))
+		    ;
 		else if (match_xsd_string(ptr, "recordPacking", o,
 				     &req->recordPacking))
 		    ;
-                else if (match_xsd_string(ptr, "version", o,
-                                           &(*p)->srw_version))
-		    ;
+		else if (match_xsd_string(ptr, "database", o,
+				     &req->database))
+                    ;
 	    }
         }
         else if (!strcmp(method->name, "explainResponse"))
@@ -553,11 +573,11 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 
 	    for (; ptr; ptr = ptr->next)
 	    {
-		if (match_element(ptr, "record"))
-		    yaz_srw_record(o, ptr, &res->record, client_data, ns);
-                else if (match_xsd_string(ptr, "version", o,
-					  &(*p)->srw_version))
+                if (match_xsd_string(ptr, "version", o,
+                                           &(*p)->srw_version))
 		    ;
+		else if (match_element(ptr, "record"))
+		    yaz_srw_record(o, ptr, &res->record, client_data, ns);
 		else if (match_element(ptr, "diagnostics"))
                     yaz_srw_diagnostics(o, ptr, &res->diagnostics,
                                         &res->num_diagnostics,
@@ -586,18 +606,18 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 		else if (match_xsd_string(ptr, "scanClause", o,
 				     &req->scanClause))
 		    ;
-		else if (match_xsd_string(ptr, "database", o,
-					  &req->database))
-                    ;
-		else if (match_xsd_string(ptr, "stylesheet", o,
-					  &req->stylesheet))
-		    ;
 		else if (match_xsd_integer(ptr, "responsePosition", o,
 					   &req->responsePosition))
 		    ;
 		else if (match_xsd_integer(ptr, "maximumTerms", o,
 					   &req->maximumTerms))
                     ;
+		else if (match_xsd_string(ptr, "stylesheet", o,
+					  &req->stylesheet))
+		    ;
+		else if (match_xsd_string(ptr, "database", o,
+					  &req->database))
+                    ;
 	    }
         }
         else if (!strcmp(method->name, "scanResponse"))
@@ -614,7 +634,10 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 	    
 	    for (; ptr; ptr = ptr->next)
 	    {
-                if (match_element(ptr, "terms"))
+		if (match_xsd_string(ptr, "version", o,
+				     &(*p)->srw_version))
+		    ;
+                else if (match_element(ptr, "terms"))
                     yaz_srw_terms(o, ptr, &res->terms,
 				  &res->num_terms, client_data,
 				  ns);
@@ -622,9 +645,6 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                     yaz_srw_diagnostics(o, ptr, &res->diagnostics,
                                         &res->num_diagnostics,
                                         client_data, ns);
-		else if (match_xsd_string(ptr, "version", o,
-					  &(*p)->srw_version))
-		    ;
 	    }
         }
         else
@@ -660,6 +680,12 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                 add_xsd_string(ptr, "pQuery", req->query.pqf);
                 break;
             }
+            add_xsd_integer(ptr, "startRecord", req->startRecord);
+            add_xsd_integer(ptr, "maximumRecords", req->maximumRecords);
+            add_xsd_string(ptr, "recordPacking", req->recordPacking);
+            add_xsd_string(ptr, "recordSchema", req->recordSchema);
+            add_xsd_string(ptr, "recordXPath", req->recordXPath);
+            add_xsd_integer(ptr, "resultSetTTL", req->resultSetTTL);
             switch(req->sort_type)
             {
             case Z_SRW_sort_type_none:
@@ -671,14 +697,8 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                 add_xsd_string(ptr, "xSortKeys", req->sort.xSortKeys);
                 break;
             }
-            add_xsd_integer(ptr, "startRecord", req->startRecord);
-            add_xsd_integer(ptr, "maximumRecords", req->maximumRecords);
-            add_xsd_string(ptr, "recordSchema", req->recordSchema);
-            add_xsd_string(ptr, "recordPacking", req->recordPacking);
-            add_xsd_string(ptr, "recordXPath", req->recordXPath);
-            add_xsd_string(ptr, "database", req->database);
-            add_xsd_integer(ptr, "resultSetTTL", req->resultSetTTL);
             add_xsd_string(ptr, "stylesheet", req->stylesheet);
+            add_xsd_string(ptr, "database", req->database);
         }
         else if ((*p)->which == Z_SRW_searchRetrieve_response)
         {
@@ -699,21 +719,26 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
                 yaz_srw_records(o, rptr, &res->records, &res->num_records,
                                 client_data, ns);
             }
+            add_xsd_integer(ptr, "nextRecordPosition",
+			    res->nextRecordPosition);
             if (res->num_diagnostics)
             {
                 xmlNodePtr rptr = xmlNewChild(ptr, 0, "diagnostics", 0);
                 yaz_srw_diagnostics(o, rptr, &res->diagnostics,
                                     &res->num_diagnostics, client_data, ns);
             }
-            add_xsd_integer(ptr, "nextRecordPosition", res->nextRecordPosition);
         }
         else if ((*p)->which == Z_SRW_explain_request)
         {
+            Z_SRW_explainRequest *req = (*p)->u.explain_request;
             xmlNodePtr ptr = xmlNewChild(pptr, 0, "explainRequest", 0);
 	    ns_srw = xmlNewNs(ptr, ns, "zs");
 	    xmlSetNs(ptr, ns_srw);
 
 	    add_xsd_string(ptr, "version", (*p)->srw_version);
+            add_xsd_string(ptr, "recordPacking", req->recordPacking);
+            add_xsd_string(ptr, "stylesheet", req->stylesheet);
+            add_xsd_string(ptr, "database", req->database);
         }
         else if ((*p)->which == Z_SRW_explain_response)
         {
@@ -747,6 +772,7 @@ int yaz_srw_codec(ODR o, void * vptr, Z_SRW_PDU **handler_data,
 	    add_xsd_integer(ptr, "responsePosition", req->responsePosition);
 	    add_xsd_integer(ptr, "maximumTerms", req->maximumTerms);
 	    add_xsd_string(ptr, "stylesheet", req->stylesheet);
+            add_xsd_string(ptr, "database", req->database);
         }
         else if ((*p)->which == Z_SRW_scan_response)
         {
