@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 1995-2003, Index Data
+ * Copyright (c) 1995-2004, Index Data
  * See the file LICENSE for details.
  *
- * $Id: unix.c,v 1.3 2003/12/30 00:29:53 adam Exp $
+ * $Id: unix.c,v 1.4 2004/04/29 21:19:23 adam Exp $
  * UNIX socket COMSTACK. By Morten Bgeskov.
  */
 #ifndef WIN32
@@ -343,9 +343,6 @@ static int unix_rcvconnect(COMSTACK h)
     return 0;
 }
 
-#define CERTF "ztest.pem"
-#define KEYF "ztest.pem"
-
 static int unix_bind(COMSTACK h, void *address, int mode)
 {
     unix_state *sp = (unix_state *)h->cprivate;
