@@ -1,6 +1,6 @@
 /*
  * Private C header for ZOOM C.
- * $Id: zoom-p.h,v 1.2 2003/12/20 00:51:19 adam Exp $
+ * $Id: zoom-p.h,v 1.4 2004/02/23 09:26:11 adam Exp $
  */
 
 #include <yaz/proto.h>
@@ -33,7 +33,7 @@ struct ZOOM_connection_p {
     char *path;
     int error;
     char *addinfo;
-    const char *diagset;
+    char *diagset;
     int state;
     int mask;
     int reconnect_ok;
@@ -151,6 +151,11 @@ struct ZOOM_task_p {
         } scan;
 #define ZOOM_TASK_PACKAGE 5
         ZOOM_package package;
+#define ZOOM_TASK_SORT 6
+	struct {
+	    ZOOM_resultset resultset;
+	    ZOOM_query q;
+	} sort;
     } u;
     ZOOM_task next;
 };
