@@ -1,14 +1,19 @@
 /*
- * $Id: zoomsh.c,v 1.24 2003/11/25 23:19:59 adam Exp $
+ * Copyright (c) 2002-2004, Index Data.
+ * See the file LICENSE for details.
  *
- * ZOOM-C Shell
+ * $Id: zoomsh.c,v 1.29 2004/04/29 08:54:56 adam Exp $
  */
 
+/* ZOOM-C Shell */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 
+#include <yaz/comstack.h>
+
 #if HAVE_READLINE_READLINE_H
 #include <readline/readline.h> 
 #endif
@@ -355,7 +360,8 @@ static void cmd_scan (ZOOM_connection *c, ZOOM_resultset *r,
             {
                 int occ = 0;
                 int len = 0;
-                const char *term = ZOOM_scanset_term(s[i], p, &occ, &len);
+                const char *term = ZOOM_scanset_display_term(s[i], p,
+				&occ, &len);
                 fwrite(term, 1, len, stdout);
                 printf (" %d\n", occ);
             }            
@@ -364,6 +370,25 @@ static void cmd_scan (ZOOM_connection *c, ZOOM_resultset *r,
     }
 }
 
+static void cmd_sort (ZOOM_connection *c, ZOOM_resultset *r,
+                      ZOOM_options options,
+                      const char **args)
+{
+    const char *sort_spec = *args;
+    int i;
+    
+    while (*sort_spec == ' ')
+        sort_spec++;
+    
+    for (i = 0; i<MAX_CON; i++)
+    {
+        if (r[i])
+            ZOOM_resultset_sort(r[i], "yaz", sort_spec);
+    }
+    while (ZOOM_event(MAX_CON, c))
+        ;
+}
+
 static void cmd_help (ZOOM_connection *c, ZOOM_resultset *r,
 		      ZOOM_options options,
 		      const char **args)
@@ -390,7 +415,7 @@ static void cmd_help (ZOOM_connection *c, ZOOM_resultset *r,
     printf (" piggyback\n");
     printf (" group\n");
     printf (" user\n");
-    printf (" pass\n");
+    printf (" password\n");
     printf (" implementationName\n");
     printf (" charset\n");
     printf (" lang\n");
@@ -475,6 +500,8 @@ static int cmd_parse (ZOOM_connection *c, ZOOM_resultset *r,
 	cmd_debug(c, r, options, buf);
     else if (is_command ("scan", cmd_str, cmd_len))
 	cmd_scan(c, r, options, buf);
+    else if (is_command ("sort", cmd_str, cmd_len))
+	cmd_sort(c, r, options, buf);
     else
 	printf ("unknown command %.*s\n", cmd_len, cmd_str);
     return 2;
